// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../ntricks.h"
#include "ce.h"

static int unused __attribute__((unused));

struct ce ce_config;

double sim_ce(struct device *in)
{
//<clean>
ce_load_config(&ce_config,in);
int i;
in->dt=ce_config.cedt;
char outpath[1000];
double extractedcharges=0.0;
double still_trapped=0.0;
FILE *train;
FILE *iv;

carrier_count_reset(in);

in->go_time=FALSE;

#ifndef mindump
train=fopena(in->outputpath,"./gex.dat","w");
fclose(train);

iv=fopena(in->outputpath,"./cen.dat","w");
fclose(iv);

FILE *check=fopen("./cecheck.dat","w");
fclose(check);
#endif


printf("Charge Extraction V= %le Sun=%le\n",ce_config.cevoltage,in->Psun);

int ittr=0;
int step=0;


struct istruct cetrapped;
inter_init(&cetrapped);

struct istruct cer;
inter_init(&cer);

struct istruct ceextracted;
inter_init(&ceextracted);

struct istruct cei;
inter_init(&cei);

double Psun_orig=in->Psun;
light_solve_and_update(in,&(in->mylight),in->Psun,0.0);

printf("Vapplied=%le\n",ce_config.cevoltage);
double trapped_in_dark=get_extracted_np(in);
ramp_externalv(in,in->Vapplied,ce_config.cevoltage);
double trapped_at_start=get_extracted_np(in);


//sim_externalv(in,ce_config.cevoltage);
//solve_all(in);
plot_now(in,"plot");


device_timestep(in);

in->go_time=TRUE;

int turnofflight=TRUE;

in->time=-ce_config.cepresteps*in->dt;
double np_start=0.0;
double np_out_tot=0.0;
double r_tot=0.0;
//double last_n=0.0;
int cestep=0;
do
{
if (step==ce_config.cepresteps) carrier_count_reset(in);

	if (step<ce_config.cepresteps)
	{
		sim_externalv(in,ce_config.cevoltage);
		//printf("external %lf\n",ce_config.cevoltage);
	}else
	{
		if (turnofflight==TRUE)
		{
			np_start=get_extracted_np(in);
			//printf("%le %le \n",(n_start+p_start)/2.0,get_extracted_np(in));
			//getchar();
			in->Psun=0;
			light_solve_and_update(in,&(in->mylight),in->Psun,0.0);
			extractedcharges=0;
			np_out_tot=0.0;
			r_tot=0.0;
			turnofflight=FALSE;
		}

		newton_charge_extraction(in);

		cestep++;
	}


	#ifndef mindump
	printf("ce time=%e step=%d %.1e ",in->time,step,in->last_error);
	printf("V=%lf current = %e mA (%e A/m^2)\n",in->Vapplied,get_I(in)/1e-3,get_J(in));

	#endif

	ittr++;

	get_avg_recom(in);

	#ifndef mindump
	plot_now(in,"plot");

	//("%le %le %le\n",(n_start+p_start)/2.0,get_extracted_np(in),fabs(np_out_tot)/2.0);

	iv=fopena(in->outputpath,"./cen.dat","a");
	fprintf(iv,"%le %le %le %le %le\n",in->time,get_free_n_charge(in),get_free_p_charge(in),get_n_trapped_charge(in),get_p_trapped_charge(in));
	fclose(iv);

	train=fopena(in->outputpath,"./gex.dat","a");
	fprintf(train,"%.20e %.20e %.20e\n",in->time,in->Gn[0],in->Gp[0]);
	fclose(train);
	#endif




	if (in->time>0.0)
	{

		extractedcharges+=(get_I_ce(in)/q)*in->dt;
		np_out_tot+=(get_I_ce(in)/q)*in->dt/in->Vol;
		r_tot+=get_avg_recom(in)*in->dt;
		still_trapped=get_extracted_np(in);

		inter_append(&ceextracted,in->time,np_out_tot);
		inter_append(&cer,in->time,r_tot);
		inter_append(&cei,in->time,get_I_ce(in));
		inter_append(&cetrapped,in->time,fabs(still_trapped));

		#ifndef mindump
		FILE *check=fopen("./cecheck.dat","a");
		fprintf(check,"%le %le %le %le %le\n",in->time,fabs(np_out_tot),fabs(r_tot),fabs(still_trapped),fabs(np_start));
		fclose(check);
		#endif

	}

	/*if ((ce_config.cefast==TRUE)&&(cestep>2))
	{
		if (fabs(last_n-np_out_tot)<1e6)
		{
			printf("ce converged\n");
			break;
		}
	}*/

	if (in->time>0.0) in->dt*=ce_config.cedtmull;
	device_timestep(in);


	step++;

	//last_n=np_out_tot;
	if (in->time>ce_config.cetimestop) break;


	if (in->stop==TRUE)
	{
		break;
	}
	//printf("%le %le\n",in->ilast,get_I(in)+in->C*(in->Vapplied-in->Vapplied_last)/in->dt+in->Vapplied/in->Rshunt);
	//getchar();
	in->ilast=get_I(in)+in->C*(in->Vapplied-in->Vapplied_last)/in->dt+in->Vapplied/in->Rshunt;

if (in->stop_start==TRUE) getchar();
}while(1);

in->go_time=FALSE;
double capcharges=0.0;
capcharges=ce_config.cevoltage*in->C/q;
capcharges/=in->Vol;

inter_save_a(&ceextracted,in->outputpath,"ceextracted.dat");

double extracted_charge=inter_get_hard(&ceextracted,20.0)-capcharges;
still_trapped=inter_get_hard(&cetrapped,20.0);
double lost_to_r=inter_get_hard(&cer,20.0);
double ce_error=100.0*extracted_charge/np_start+100.0*lost_to_r/np_start+100.0*still_trapped/np_start;
printf("CE stats\n");
printf("-----------\n");
printf("extracted = %lf %%\n",100.0*extracted_charge/np_start);
printf("recombined = %lf %%\n",100.0*lost_to_r/np_start);
printf("trapped = %lf %%\n",100.0*still_trapped/np_start);
printf("tot = %lf %%\n",ce_error);
printf("in dark=%le before run=%le at end=%le\n",trapped_in_dark,trapped_at_start,get_extracted_np(in));
printf("-----------\n");


FILE *info;
info=fopen("./ceinfo.dat","w");
fprintf(info,"%le %le %le %le %le %le %le\n",Psun_orig,ce_config.cevoltage,extracted_charge,lost_to_r,still_trapped,np_start,ce_error);
fclose(info);

info=fopen("./cen_tot.dat","w");
fprintf(info,"%le %le\n",ce_config.cevoltage,extractedcharges);
fclose(info);

inter_save_a(&cer,in->outputpath,"cer.dat");
inter_free(&cer);

inter_save_a(&cei,in->outputpath,"cei_nomod.dat");
inter_mod(&cei);
inter_save_a(&cei,in->outputpath,"cei.dat");

inter_save_a(&cetrapped,in->outputpath,"cetrapped.dat");
inter_free(&cetrapped);

if (in->dumpitdos==TRUE)
{
	inter_chop(&cei,1e-9, 1.0);

	for (i=0;i<cei.len;i++)
	{
	cei.data[i]*=cei.x[i];
	cei.x[i]=(25e-3*log(2e7*cei.x[i]));
	}

	double val=inter_get_hard(&cei,0.0);

	inter_rescale(&cei,1.0, 1.0/val);

	sprintf(outpath,"%s%s",in->outputpath,"cei_itdos.dat");
	inter_save(&cei,outpath);

}

inter_free(&cei);

inter_free(&ceextracted);

return extractedcharges;
//</clean>
}

void ce_load_config(struct ce *in,struct device *dev)
{
double ver=0.0;
char name[200];
FILE *config=fopena(dev->outputpath,"ce.inp","r");

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->cedtmull));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->cetimestop));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->cevoltage));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->ceshortvoltage));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d ",&(in->cefast));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le ",&(in->cedt));
unused=fscanf(config,"%s",name);
unused=fscanf(config,"%d ",&(in->cepresteps));

unused=fscanf(config,"%s",name);
unused=fscanf(config,"%le",&(in->ce_L));


unused=fscanf(config,"%s",name);
unused=fscanf(config,"%lf",&(ver));
if (ver!=1.0)
{
ewe("File compatability problem %s\n","ce.inp");
}

unused=fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
ewe("Problem with ce.inp file last item read %s!\n",name);
}
fclose(config);
}
