// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <frame.h>
#include "sim.h"
#include "slice.h"
#include "dump.h"

static int unused __attribute__((unused));

struct slices sl;

void slice_init(struct device* in)
{
#ifdef enable_time
int i=0;
FILE *config;
config=fopena(in->outputpath,"dump_time.inp","r");
if (config==NULL)
{
	printf("dump_time.inp not found\n");
	exit(0);
}

unused=fscanf(config,"%d",&(sl.number));
for (i=0;i<sl.number;i++)
{
	unused=fscanf(config,"%le",&(sl.time[i]));
	sl.done[i]=FALSE;
}
fclose(config);
sl.dump_number=0;

config=fopena(in->outputpath,"dump_slice_info.dat","w");
fclose(config);
#endif
}

void slice_check(struct device* in)
{
#ifdef enable_time
	FILE* out;
	int i=0;
	int dump=FALSE;
	int number=0;

	if (get_dump_status(dump_slices_by_time)==TRUE)
	{

		for (i=0;i<sl.number;i++)
		{
			if ((in->time>sl.time[i])&&(sl.done[i]==FALSE))
			{
				sl.done[i]=TRUE;
				number=i;
				dump=TRUE;
				break;
			}
		}
	}

	if (get_dump_status(dump_all_slices)==TRUE)
	{
		number=sl.dump_number;
		sl.dump_number++;
		dump=TRUE;
	}

	if (dump==TRUE)
	{
		char prefix[100];
		sprintf(prefix,"_%d",number);
		dump_for_plot(in,prefix);//dump_slice(in,prefix);

		#ifdef private
		if (get_dump_status(dump_energy_slice_switch)==TRUE)
		{
				dump_energy_slice(in,number,in->dump_slicepos);
		}
		#endif

		printf("Dumped time slice\n");
		out=fopena(in->outputpath,"dump_slice_info.dat","a");
		fprintf(out,"%.4lf %le\n",get_equiv_V(in),in->time);
		fclose(out);
	}

#endif
}

