# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import pygtk
pygtk.require('2.0')
import gtk
import sys
import os
import shutil
from scan_item import scan_item
from scan_item import scan_item_add
from util import find_data_file
from numpy import *
from matplotlib.figure import Figure
from numpy import arange, sin, pi
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
from matplotlib import ticker
import matplotlib.cm as cm
from matplotlib.mlab import griddata
import glob
from plot import plot_data
from matplotlib import rcParams

class cmp_class(gtk.Window):
	mix_y=None
	max_y=None
	max_z=1e24
	mygraph=plot_data()
	icon_theme = gtk.icon_theme_get_default()
	def check_2d_file(self,name):
		mapfiles=["pt_map","nt_map"]
		filename=os.path.basename(name)
		for i in range(len(filename)-1,0,-1):
			if filename[i]=="_":
				break
		data=filename[:i]
		count=mapfiles.count(data)
		if count==1:
			return True
		else:
			return False

	def callback_close(self, widget, data=None):
		self.hide()
		return True

	def load_data_file(self):
		try:

			f = open("dump_slice_info.dat")
			self.lines = f.readlines()
			f.close()

			for i in range(0, len(self.lines)):
				self.lines[i]=self.lines[i].rstrip()
			return True

		except IOError as e:
			self.lines=["none"]
			print "Can not open","dump_slice_info.dat"
			return False

	def set_min_max(self):
		my_max=-1e38
		my_min=1e38
		files=self.entry2.get_text().split()
		path0=self.entry0.get_text()
		my_z_max=0.0
		my_z_min=1e30
		for ii in range(0,len(files)):
			for i in range(0,int(self.adj1.get_upper())):
				file_name=files[ii]+"_"+str(i)+".dat"

				if self.check_2d_file(file_name)==True:
					try:
						x,y,z = loadtxt(path0+file_name, unpack=True)
						min_=min(y)
						max_=max(y)
						max_z=max(z)
						min_z=min(z)

						if (min_<my_min):
							my_min=min_
						if (max_>my_max):
							my_max=max_
						if (max_z>my_z_max):
							my_z_max=max_z
						if (min_z<my_z_min):
							my_z_min=min_z
					except:
						print "no file"
				else:
					try:
						x,y = loadtxt(path0+file_name, unpack=True)

						min_=min(y)
						max_=max(y)
						if (min_<my_min):
							my_min=min_
						if (max_>my_max):
							my_max=max_
					except:
						print "not found"
		self.mix_y=my_min
		self.max_y=my_max
		self.max_z=my_z_max
		self.min_z=my_z_min

	def update(self,files,value):

		value=int(value)
		if value>len(self.lines) or self.lines[0]=="none":
			md = gtk.MessageDialog(self, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING,  gtk.BUTTONS_CLOSE, "Re-run the simulation with 'dump all slices' set to one to use this tool.")
        		md.run()
        		md.destroy()
			return
		ret=self.mygraph.find_file(files[0]+".dat",None)
		if ret==False:
			return

		self.fig.clf()
		self.ax1 = self.fig.add_subplot(111, axisbg='white')
		self.fig.subplots_adjust(bottom=0.2)
		title=self.lines[value].split()
		out="Voltage="+title[0]+" time="+title[1]
		self.fig.suptitle(out)
		if self.mix_y!=None:
			self.ax1.set_ylim([self.mix_y,self.max_y])
		self.ax1.set_xlabel(self.mygraph.read.x_label)
		self.ax1.set_ylabel(self.mygraph.read.y_label)


		color =['r','g','b','y','o','r','g','b','y','o']
		color2 =[ cm.Reds,cm.Blues,cm.Greens,cm.Reds,cm.Blues,cm.Greens]
		path0=self.entry0.get_text()
		path1=self.entry1.get_text()

		graph_pointer=[]
		for i in range(0,len(files)):
			file_name=files[i]+"_"+str(value)+".dat"

			if self.check_2d_file(file_name)==True:
				try:
					x,y,z = loadtxt(path0+file_name, unpack=True)
					#x=x*1e9
					#y=y*float(self.mygraph.read.y_mul)
					print self.mygraph.read.x_mul,self.mygraph.read.y_mul
					xi = linspace(min(x), max(x))
					yi = linspace(min(y), max(y))

					X, Y = meshgrid(xi, yi)
					Z = griddata(x, y, z, xi, yi)
					#print self.min_z,self.max_z
					surf = self.ax1.contourf(xi,yi,Z,locator=ticker.LogLocator(),cmap=color2[i],alpha=0.7)
					if i==0:
						self.fig.colorbar(surf)
				except:
					print "no file",path0+file_name
			else:
				try:
					x,y = loadtxt(path0+file_name, unpack=True)
					x=x*+self.mygraph.read.x_mul
					y=y*+self.mygraph.read.y_mul

					if (self.mygraph.read.logscale_x==1):
						self.ax1.set_xscale("log")

					if (self.mygraph.read.logscale_y==1):
						self.ax1.set_yscale("log")

					Ec, = self.ax1.plot(x,y, color[i], linewidth=3 ,alpha=0.5)
					print "added key"
					graph_pointer.append(Ec)
				except:
					print "File ",file_name,"not found"

				if path0 != path1:
					try:
						x,y = loadtxt(path1+file_name, unpack=True)
						x=x*+self.mygraph.read.x_mul
						y=y*+self.mygraph.read.y_mul
						Ec, = self.ax1.plot(x,y, color[i]+"o-", linewidth=3 ,alpha=0.5)
					except:
						print "File ",file_name,"not found"

		self.fig.legend(graph_pointer, files, 'upper right')
		self.fig.canvas.draw()

	def callback_scale(self, adj):
		self.update(self.entry2.get_text().split(),adj.value)
		#["n","nt","p","pt"]["Ec","Ev","Fn","Fp"]

	def callback_auto_scale(self, adj):
		self.set_min_max()
		self.update(self.entry2.get_text().split(),self.adj1.value)

	def callback_edit(self,data):
		print os.getcwd()
		a = open("./gui_cmp_config.inp", "w")
		a.write(self.entry0.get_text()+"\n")
		a.write(self.entry1.get_text()+"\n")
		a.write(self.entry2.get_text()+"\n")
		a.close()
		self.update_slider()
		self.update(self.entry2.get_text().split(),self.adj1.value)
		print "Saved"

	def config_load(self):
		try:
	 		f = open("./gui_cmp_config.inp", "r")
			lines = f.readlines()
			f.close()

			for i in range(0, len(lines)):
				lines[i]=lines[i].rstrip()

			self.entry0.set_text(lines[0])
			self.entry1.set_text(lines[1])
			self.entry2.set_text(lines[2])
		except:
			self.entry0.set_text("./snapshots/")
			self.entry1.set_text("./")
			self.entry2.set_text("pt_map nt_map")

	def update_slider(self):
		files=glob.glob(self.entry0.get_text()+'/Ec_*.dat')
		self.adj1.set_upper(len(files))

	def save_image(self,file_name):
		fileName, ext = os.path.splitext(file_name)

		if (ext==".jpg"):
			self.canvas.figure.savefig(file_name)
		elif ext==".avi":
			out_file="./snapshots/"
			jpgs=""
			for i in range(0,int(self.adj1.get_upper())):
				self.update(self.entry2.get_text().split(),i)
				image_name=out_file+"image_"+str(i)+".jpg"
				self.canvas.figure.savefig(image_name)
				jpgs=jpgs+" mf://"+image_name

			os.system("mencoder "+jpgs+" -mf type=jpg:fps=1.0 -o "+file_name+" -ovc lavc -lavcopts vcodec=mpeg1video:vbitrate=800")
			#msmpeg4v2
		else:
			print "Unknown file extention"

	def callback_save(self, widget, data=None):
		dialog = gtk.FileChooserDialog("Save as..",
                               None,
                               gtk.FILE_CHOOSER_ACTION_SAVE,
                               (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                gtk.STOCK_SAVE, gtk.RESPONSE_OK))
		dialog.set_default_response(gtk.RESPONSE_OK)

		filter = gtk.FileFilter()
		filter.set_name(".jpg")
		filter.add_pattern("*.jpg")
		dialog.add_filter(filter)

		filter = gtk.FileFilter()
		filter.set_name(".avi")
		filter.add_pattern("*.avi")
		dialog.add_filter(filter)

		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			dialog.hide()
			file_name=dialog.get_filename()

			print os.path.splitext(file_name)[1]
			if os.path.splitext(file_name)[1]:
				self.save_image(file_name)
			else:
				filter=dialog.get_filter()
				self.save_image(file_name+filter.get_name())

		elif response == gtk.RESPONSE_CANCEL:
		    print 'Closed, no files selected'
		dialog.destroy()

	def callback_help(self, widget, data=None):
		cmd = 'firefox http://www.roderickmackenzie.eu/opvdm_wiki.html'
		os.system(cmd)

	def init(self):
		ret=self.load_data_file()
		if ret==False:
			return False
		vbox=gtk.VBox()

		toolbar = gtk.Toolbar()
		toolbar.set_style(gtk.TOOLBAR_ICONS)
		toolbar.set_size_request(-1, 50)

		tool_bar_pos=0
	        image = gtk.Image()
   		image.set_from_file(self.icon_theme.lookup_icon("video", 32, 0).get_filename())
		self.video = gtk.ToolButton(image)
		toolbar.insert(self.video, tool_bar_pos)
		self.video.connect("clicked", self.callback_save)
		tool_bar_pos=tool_bar_pos+1

	        image = gtk.Image()
   		image.set_from_file(find_data_file("gui/scale.png"))
		self.scale = gtk.ToolButton(image)
		toolbar.insert(self.scale, tool_bar_pos)
		self.scale.connect("clicked", self.callback_auto_scale)
		tool_bar_pos=tool_bar_pos+1

		sep = gtk.SeparatorToolItem()
		sep.set_draw(False)
		sep.set_expand(True)
		toolbar.insert(sep, tool_bar_pos)
		sep.show()
		tool_bar_pos=tool_bar_pos+1


		help = gtk.ToolButton(gtk.STOCK_HELP)
		toolbar.insert(help, tool_bar_pos)
		help.connect("clicked", self.callback_help)
		help.show()
		tool_bar_pos=tool_bar_pos+1

		close = gtk.ToolButton(gtk.STOCK_QUIT)
		close.connect("clicked", self.callback_close)
		toolbar.insert(close, tool_bar_pos)
		close.show()
		tool_bar_pos=tool_bar_pos+1


		toolbar.show_all()
		vbox.pack_start(toolbar, False, False, 10)



		#canvas
		self.fig = Figure(figsize=(5,4), dpi=100)
		self.canvas = FigureCanvas(self.fig)  # a gtk.DrawingArea
		self.canvas.figure.patch.set_facecolor('white')
		self.canvas.set_size_request(700, 300)
		vbox.pack_start(self.canvas, True, True, 10)

		self.canvas.show()
		vbox.add(self.canvas)



		#adjust
		self.adj1 = gtk.Adjustment(0.0, 0.0, 100, 1, 1.0, 1.0)
		self.adj1.connect("value_changed", self.callback_scale)
		vscale = gtk.HScale(self.adj1)
		vscale.set_update_policy(gtk.UPDATE_CONTINUOUS)
		vscale.set_digits(1)
		vscale.set_value_pos(gtk.POS_TOP)
		vscale.set_draw_value(True)
		vscale.set_size_request(200, 40)
		vscale.set_digits(0)
		vbox.pack_start(vscale, False, False, 10)


		hbox=gtk.HBox()
		text=gtk.Label("Primary dir")
		hbox.add(text)
		self.entry0 = gtk.Entry()
		self.entry0.set_text("./snapshots/")
		self.entry0.show()
		hbox.add(self.entry0)
		text=gtk.Label("Secondary dir")
		hbox.add(text)
		self.entry1 = gtk.Entry()
		self.entry1.set_text("./")
		self.entry1.show()
		hbox.add(self.entry1)

		hbox.show()
		hbox.set_size_request(-1, 30)
		vbox.pack_start(hbox, False, False, 10)

		hbox2=gtk.HBox()
		text=gtk.Label("Files to plot")
		hbox2.add(text)
		self.entry2 = gtk.Entry()
		self.entry2.set_text("pt_map nt_map")
		self.entry2.show()

		self.config_load()
		self.update_slider()

		self.entry0.connect("changed", self.callback_edit)
		self.entry1.connect("changed", self.callback_edit)
		self.entry2.connect("changed", self.callback_edit)

		hbox2.add(self.entry2)
		hbox2.set_size_request(-1, 30)
		vbox.pack_start(hbox2, False, False, 10)

		vbox.show_all()
		self.add(vbox)
		self.update(self.entry2.get_text().split(),0)
		#["Ec","Ev","Fn","Fp"]
		self.set_border_width(10)
		self.set_title("Compare")
		self.set_icon_from_file(find_data_file("gui/image.jpg"))
		self.show()


