# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import pygtk
pygtk.require('2.0')
import gtk
import sys
import os
import shutil
from token_lib import tokens
from util import pango_to_gnuplot
def load_graph(path):
	cmd = '/usr/bin/gnuplot -persist '+path
	print cmd
	os.system(cmd)

def check_info_file(search_string):
	files=["dos0.inp","photokit_ri.dat","photokit_real.dat","photokit_imag.dat","jv_psun_voc.dat","jv_voc_nt.dat","jv_voc_pt.dat","jv_psun_nf_nt.dat","jv_psun_pf_pt.dat","jv_psun_np_tot.dat","sim_info.dat"]
	if files.count(search_string)> 0:
		return True
	else:
		return False


class plot_info():
	file_name=""
	x_label=""
	y_label=""
	other_file=""
	x_mul=1.0
	y_mul=1.0
	logscale_x=0
	logscale_y=0
	title=""
	def __init__(self,a,b,c,d,e,f,g,h,i):
		self.file_name=a
		self.x_label=b
		self.y_label=c
		self.x_mul=d
		self.y_mul=e
		self.other_file=f
		self.logscale_x=g
		self.logscale_y=h
		self.title=i



class plot_data:

	files=[]
	pos=0
	def __init__(self):
		self.files.append(plot_info("celiv_i.dat","Time (us)","Current (Amps)",1e6,1.0,"none",0,0,"CELIV - Time - Current"))
 		self.files.append(plot_info("alpha.inp","Wavelength (nm)","Absorption (m^{-1})",1e9,1.0,"n.inp",0,0,"Absorption"))
 		self.files.append(plot_info("n.inp","Wavelength (nm)","Refractive index (au)",1e9,1.0,"alpha.inp",0,0,"Refractive index"))
		self.files.append(plot_info("ivexternal.dat","Applied Bias (Volts)","Current (A)",1.0,1.0,"none",0,0,"Current - Voltage"))
		self.files.append(plot_info("jvexternal.dat","Applied Bias (Volts)","Current Density (Am^{-2})",1.0,1.0,"none",0,0,"Current Density - Voltage"))
		self.files.append(plot_info("charge.dat","Applied Bias (Volts)","Charge density (m^{-3})",1.0,1.0,"none",0,0,"Applied Bias - Charge density"))
		self.files.append(plot_info("tof_i.dat","Time (s)","Current (A)",1.0,1.0,"none",1,1,"ToF - Time - Current"))
		self.files.append(plot_info("tof_dos.dat","Energy (eV)","Magnitude (au)",1.0,1.0,"none",0,1,"ToF extracted DoS i*t"))
		self.files.append(plot_info("pulse_i_pos.dat","Time (us)","Current (A)",1e6,1.0,"none",0,0,"ToF - Time - |Current|"))
		self.files.append(plot_info("pulse_i.dat","Time (us)","Current (A)",1e6,1.0,"none",0,0,"Time - Current"))
		self.files.append(plot_info("pulse_i_norm.dat","Time (us)","Current (au)",1e6,1.0,"none",0,0,"Time - |Current|"))
		self.files.append(plot_info("pulse_v.dat","Time (us)","Voltage (Volts)",1e6,1.0,"none",0,0,"Time - voltage"))
		self.files.append(plot_info("pulse_mue.dat","Time (us)","Electron Mobility (m^{2}V^{-1}s^{-1})",1e6,1.0,"none",0,0,"Time - electon mobility"))
		self.files.append(plot_info("pulse_muh.dat","Time (us)","Hole Mobility (m^{2}V^{-1}s^{-1})",1e6,1.0,"none",0,0,"Time - hole mobility"))
		self.files.append(plot_info("photokit_i.dat","Time (us)","Current (A)",1e6,1.0,"photokit_psun.dat",0,0,"Time - Current"))
		self.files.append(plot_info("photokit_i_norm.dat","Time (us)","Current (au)",1e6,1.0,"none",0,0,"Time - |Current|"))
		self.files.append(plot_info("photokit_v.dat","Time (us)","Voltage (Volts)",1e6,1.0,"none",0,0,"Time - Voltage"))
		self.files.append(plot_info("photokit_mue.dat","Time (us)","Electron Mobility (m^{2}V^{-1}s^{-1})",1e6,1.0,"none",0,0,"Time - Electron Mobility"))
		self.files.append(plot_info("photokit_muh.dat","Time (us)","Hole Mobility (m^{2}V^{-1}s^{-1})",1e6,1.0,"none",0,0,"Time - Hole Mobility"))
		self.files.append(plot_info("photokit_psun.dat","Time (us)","Suns (Suns)",1e6,1e-3,"none",0,0,"Time - Sun Intensity"))
		self.files.append(plot_info("photokit_ri.dat","Re - Amps","Im - Amps",1.0,1.0,"none",0,0,"Re(I) - Im(I)"))
		self.files.append(plot_info("photokit_real.dat","Hz","Re - Amps",1.0,1.0,"none",1,0,"Frequency - Re(I)"))
		self.files.append(plot_info("photokit_imag.dat","Hz","Im - Amps",1.0,1.0,"none",1,0,"Frequency - Im(I)"))
		self.files.append(plot_info("photokit_i_norm_x.dat","Time","Current (A)",1.0,1.0,"none",0,0,"Wave scaled"))
		self.files.append(plot_info("pulse_voc_v.dat","Time (us)","Voltage (Volts)",1e6,1.0,"none",0,0,"Time - voltage"))
		self.files.append(plot_info("mu_n.dat","Position (nm)","Electron Mobility (m^{2}V^{-1}s^{-1})",1e9,1.0,"none",0,0,"Position - Electron Mobility"))
		self.files.append(plot_info("mu_p.dat","Position (nm)","Hole Mobility (m^{2}V^{-1}s^{-1})",1e9,1.0,"none",0,0,"Position - Hole Mobility"))
		self.files.append(plot_info("mu_n_ft.dat","Position (nm)","Average electron mobility (m^{2}V^{-1}s^{-1})",1e9,1.0,"none",0,0,"Position - Electron Mobility"))
		self.files.append(plot_info("mu_p_ft.dat","Position (nm)","Average hole mobility (m^{2}V^{-1}s^{-1})",1e9,1.0,"none",0,0,"Position - Hole Mobility"))
		self.files.append(plot_info("dynamic_ntrap.dat","Time (us)","Electron density (m^{-3}",1e6,1.0,"dynamic_ptrap.dat",0,0,"Time - Trapped electron density"))
		self.files.append(plot_info("dynamic_ptrap.dat","Time (us)","Hole density (m^{-3}",1e6,1.0,"dynamic_ntrap.dat",0,0,"Time - Trapped hole density"))
		self.files.append(plot_info("dynamic_nfree.dat","Time (us)","Electron density (m^{-3}",1e6,1.0,"dynamic_pfree.dat",0,0,"Time - Free electron density"))
		self.files.append(plot_info("dynamic_pfree.dat","Time (us)","Hole density (m^{-3}",1e6,1.0,"dynamic_nfree.dat",0,0,"Time - Free hole density"))
		self.files.append(plot_info("dynamic_np.dat","Time (us)","Charge density (m^{-3}",1e6,1.0,"none",0,0,"Time - Charge in device (- equlibrium charge)"))
		self.files.append(plot_info("dynamic_np_norm.dat","Time (us)","Charge due to photogeneration",1e6,1.0,"none",0,0,"Time - Charge in device"))

		self.files.append(plot_info("dynamic_filledn.dat","Time (us)","Filled electron traps (Percent)",1e6,1.0,"none",0,0,"Time - Filled electron traps"))
		self.files.append(plot_info("dynamic_filledp.dat","Time (us)","Filled hole traps (Percent)",1e6,1.0,"none",0,0,"Time - Filled hole traps"))


		self.files.append(plot_info("dynamic_Rn.dat","Time (us)","Electron recombination rate",1e6,1.0,"none",0,0,"Time - Electron recombination rate"))

		self.files.append(plot_info("dynamic_Rp.dat","Time (us)","Hole recombination rate",1e6,1.0,"none",0,0,"Time - hole recombination rate"))

		self.files.append(plot_info("jv_psun_voc.dat","Light intensity (Suns)","V_{oc} (Volts)",1e-3,1.0,"none",0,0,"Light intensity - V_{oc}"))

		self.files.append(plot_info("jv_voc_nt.dat","V_{oc} (V)","Trapped electron density (m^{-3})",1.0,1.0,"none",0,0,"V_{oc} - Trapped Electrons"))

		self.files.append(plot_info("jv_voc_pt.dat","V_{oc} (V)","Trapped hole density (m^{-3})",1.0,1.0,"none",0,0,"V_{oc} - Trapped Holes"))

		self.files.append(plot_info("jv_psun_nf_nt.dat","Suns","nf/nt",1.0,1.0,"none",0,0,"V_{oc} - Trapped Electrons"))

		self.files.append(plot_info("jv_psun_pf_pt.dat","Suns","pf/pt",1.0,1.0,"none",0,0,"V_{oc} - Trapped Holes"))

		self.files.append(plot_info("jv_psun_np_tot.dat","Suns","total charge",1.0,1.0,"none",0,0,"Sun - Total charge"))

		self.files.append(plot_info("equ_Ec.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"LUMO - position (equilibrium)"))
		self.files.append(plot_info("equ_Ev.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"HOMO - position (equilibrium)"))
		self.files.append(plot_info("equ_Fn.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"Electron Fermi level - position (equilibrium)"))
		self.files.append(plot_info("equ_Fp.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"Hole Fermi level - position (equilibrium)"))


		self.files.append(plot_info("Ec.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"LUMO - position"))
		self.files.append(plot_info("Ev.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"HOMO - position"))
		self.files.append(plot_info("Fn.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"Electron Fermi level - position"))
		self.files.append(plot_info("Fp.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"Hole Fermi level - position"))
		self.files.append(plot_info("nt_map.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"Electron density - Energy - position"))
		self.files.append(plot_info("pt_map.dat","Position (nm)","Energy (eV)",1e9,1.0,"none",0,0,"Electron density - Energy - position"))
		self.files.append(plot_info("n.dat","Position (nm)","Electron density (m^{-3})",1e9,1.0,"none",0,1,"Electron density - position"))
		self.files.append(plot_info("nt.dat","Position (nm)","Electron density (m^{-3})",1e9,1.0,"none",0,1,"Electron density - position"))
		self.files.append(plot_info("p.dat","Position (nm)","Hole density (m^{-3})",1e9,1.0,"none",0,1,"Hole density - position"))
		self.files.append(plot_info("pt.dat","Position (nm)","Hole density (m^{-3})",1e9,1.0,"none",0,1,"Hole density - position"))
		self.files.append(plot_info("Jn.dat","Position (nm)","Electron Flux (m^{-2}s^{-1})",1e9,1.0,"none",0,0,"Electron Flux - position"))
		self.files.append(plot_info("Jp.dat","Position (nm)","Hole Flux (m^{-2}s^{-1})",1e9,1.0,"none",0,0,"Hole Flux - position"))
		self.files.append(plot_info("Rn.dat","Position (nm)","Electron Recombination rate (m^{-3}s^{-1})",1e9,1.0,"none",0,0,"Electron recombination rate - position"))
		self.files.append(plot_info("Rp.dat","Position (nm)","Electron Recombination rate (m^{-3}s^{-1})",1e9,1.0,"none",0,0,"Electron recombination rate - position"))

		self.files.append(plot_info("pl.dat","Wavelength (nm)","PL Intensity(m^{-3}s^{-1})",1e9,1.0,"none",0,0,"PL intensity - position"))

	def find_file(self,name,plot_token):

		for i in range(0, len(self.files)):
			#print "'",self.files[i].file_name,"'",name.strip(),"'"
			if self.files[i].file_name==name.strip():
				self.read=self.files[i]
				return True
		print "Can not find",name,"in xy data lib"

		#check to see if I have been provided with a token
		if plot_token!=None:
			my_token_lib=tokens()
			result0=my_token_lib.find(plot_token.tag0)
			result1=my_token_lib.find(plot_token.tag1)

			if result0!=False:
				ret=plot_info("","","",1.0,1.0,"none",0,0,"")
				ret.x_label=result0.info+" ("+result0.units+")"
				ret.y_label=result1.info+" ("+result1.units+")"
				ret.title=result0.info+" "+result1.info

				print "Found tokens",plot_token.tag0,plot_token.tag1
				self.read=ret
				return True
			else:
				print "Tokens not found",plot_token.tag0,plot_token.tag1

		return False


def plot_move(dest_dir):
	f = open("./plot/temp.plot")
	lines = f.readlines()
	f.close()

	for i in range(0, len(lines)):
		lines[i]=lines[i].rstrip()

	lines[0]="set term postscript eps enhanced color \"Helvetica\" 28\n"

	if os.path.isdir(dest_dir)==False:
		print "Made new dir",dest_dir
		os.makedirs(dest_dir)


	a = open(dest_dir+'/plot.plot', "w")
	for i in range(0, len(lines)):
		a.write(lines[i]+"\n")
	a.close()

	makefile_string="main:\n"
	makefile_string=makefile_string+"\tgnuplot plot.plot >plot.eps\n"
	makefile_string=makefile_string+"\tgs -dNOPAUSE -r600 -dEPSCrop -sDEVICE=jpeg -sOutputFile=plot.jpg plot.eps -c quit\n"
	makefile_string=makefile_string+"\tgnome-open plot.jpg\n"

	text_file = open(dest_dir+"/makefile", "w")
	text_file.write(makefile_string)
	text_file.close()

def plot_gen(input_files,plot_token):
	gen_plot=True
	if (len(input_files)==1):
		if os.path.splitext(input_files[0])[1]==".plot":
			plot_file=input_files[0]
			gen_plot=False

	if gen_plot==True:
		plot_file="./plot/temp.plot"
		mygraph=plot_data()
		mygraph2=plot_data()
		mygraph.find_file(os.path.basename(input_files[0]),plot_token)
		plot_string="set terminal wxt title '"+mygraph.read.title+" - www.opvdm.com'\n"
		plot_string=plot_string+"set key top left\n"
		plot_string=plot_string+"set xlabel '"+mygraph.read.x_label+"'\n"
		plot_string=plot_string+"set ylabel '"+mygraph.read.y_label+"'\n"
		plot_string=plot_string+"set format y '%.1le%T'\n"

		if (mygraph.read.logscale_y==1):
			plot_string=plot_string+"set logscale y\n"

		if (mygraph.read.logscale_x==1):
			plot_string=plot_string+"set logscale x\n"

		y2plot=False
		if mygraph.read.other_file!="none":
			if (len(input_files)==1):
				y2plot=True

				mygraph2.find_file(mygraph.read.other_file,plot_token)
				plot_string=plot_string+"set y2label '"+mygraph2.read.y_label+"'\n"
				plot_string=plot_string+"set y2tics\n"

		plot_string=plot_string+"plot \\\n"
		print "hello",input_files
		for i in range(0, len(input_files)):
			line_name=os.path.basename(os.path.dirname(input_files[i]))
			plot_string=plot_string+"'"+input_files[i]+"' using ($1*"+str(mygraph.read.x_mul)+"):($2*"+str(mygraph.read.y_mul)+") with lp lw 2 title '"+line_name+"',\\\n"

		plot_string=plot_string[:-3]

		if y2plot==True:
			plot_string=plot_string+",\\\n'"+ os.path.dirname(input_files[0])+"/"+mygraph2.read.file_name+"' using ($1*"+str(mygraph2.read.x_mul)+"):($2*"+str(mygraph2.read.y_mul)+") axis x1y2 with lp lw 2 title '"+mygraph2.read.file_name+"'"

		text_file = open(plot_file, "w")
		text_file.write(plot_string)
		text_file.close()
	else:
		plot_file=input_files[0]

	cmd = 'gnuplot -persist '+plot_file
	os.system(cmd)
