# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import pygtk
pygtk.require('2.0')
import gtk
import sys
import os
import shutil
import signal
import subprocess
from tempfile import mkstemp
import logging

inp_dir='/usr/share/opvdm/'

def dlg_get_text( message, default=''):

	d = gtk.MessageDialog(None,
	          gtk.DIALOG_MODAL ,
	          gtk.MESSAGE_QUESTION,
	          gtk.BUTTONS_OK_CANCEL,
	          message)
	entry = gtk.Entry()
	entry.set_text(default)
	entry.show()
	d.vbox.pack_end(entry)
	entry.connect('activate', lambda _: d.response(gtk.RESPONSE_OK))
	d.set_default_response(gtk.RESPONSE_OK)

	r = d.run()
	text = entry.get_text().decode('utf8')
	d.destroy()
	if r == gtk.RESPONSE_OK:
		return text
	else:
		return None

def pango_to_gnuplot(data):
	one=""
	data.replace("<sub>", "_{")
	data.replace("</sub>", "}")

def find_data_file(name):
	local_file="./"+name
	if os.path.isfile(local_file)==True:
		ret=local_file
	else:
		ret=inp_dir+"/"+name
	return ret

def opvdm_clone():
	logging.info('opvdm_clone')
	src=get_orig_inp_file_path()
	source = os.listdir(src)
	destination = "./"
	for files in source:
		if files.endswith(".inp"):
			print "copying",files,destination
			shutil.copy(src+files,destination)


	shutil.copytree(src+"/plot", "./plot")
	shutil.copytree(src+"/exp", "./exp")
	shutil.copytree(src+"/phys", "./phys")

def set_exe_command():

	if os.path.isfile("./go.o")==True:
		exe_command=os.getcwd()+"/go.o"
		exe_name="go.o"
	else:
		exe_command="opvdm_core"
		exe_name="opvdm_core"
	return exe_command, exe_name

def get_orig_inp_file_path():

	if os.path.isfile("sim.inp")==True:
		path=os.getcwd()+"/"
	else:
		path="/usr/share/opvdm/"
	return path

def replace(file_path, token, replace):
	if os.access(file_path, os.W_OK)==False:
		print "I don't have access to "+file_path
		return

	token=token+"\n"
	replace=replace+"\n"
	fh, abs_path = mkstemp()
	new_file = open(abs_path,'w')
	old_file = open(file_path)
	skip=False

	for line in old_file:

		if skip == False:
			if line != token:
				new_file.write(line)
			else:
				new_file.write(line)
				new_file.write(replace)
				skip=True
		else:
			skip=False

	new_file.close()
	os.close(fh)
	old_file.close()
	os.remove(file_path)
	shutil.move(abs_path, file_path)

def get_token_value(file_path, token):
	old_file = open(file_path)

	output=False


	for line in old_file:
		line=line.rstrip()
		if output==True:
			old_file.close()
			return line

		if line==token:
			output=True

	old_file.close()
	return "0"


