// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <util.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <true_false.h>
#include "dump_ctrl.h"
#include "light.h"


void light_dump(struct light *in)
{
char out_dir[1024];
if ((get_dump_status(dump_optics_verbose)==TRUE)&&(in->Gn[0]!=0.0))
{
	sprintf(out_dir,"%s/light_dump/",in->output_path);
	struct stat st = {0};

	if (stat(out_dir, &st) == -1)
	{
	    mkdir(out_dir, 0700);
	}

	FILE *out;
	int i;
	int ii;
	out=fopena(out_dir,"light_2d_Ep.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],pow(pow(in->Ep[i][ii],2.0)+pow(in->Epz[i][ii],2.0),0.5));
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_2d_En.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],pow(pow(in->En[i][ii],2.0)+pow(in->Enz[i][ii],2.0),0.5));
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_2d_E_mod.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],pow(pow(in->Ep[i][ii]+in->En[i][ii],2.0)+pow(in->Enz[i][ii]+in->Epz[i][ii],2.0),1.0));
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_2d_photons.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->photons[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_2d_photons_asb.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->photons_asb[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);


	out=fopena(out_dir,"light_2d_n.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->n[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);

	out=fopena(out_dir,"light_lambda_sun.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		fprintf(out,"%le %le\n",in->l[i],in->sun[i]);
	}
	fclose(out);

	out=fopena(out_dir,"light_lambda_sun_norm.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		fprintf(out,"%le %le\n",in->l[i],in->sun_norm[i]);
	}
	fclose(out);

	out=fopena(out_dir,"light_lambda_sun_photons.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		fprintf(out,"%le %le\n",in->l[i],in->sun_photons[i]);
	}
	fclose(out);

	out=fopena(out_dir,"light_lambda_alpha.dat","w");
	for (i=0;i<in->lpoints;i++)
	{
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le %le\n",in->l[i],in->x[ii],in->alpha[i][ii]);
		}

	fprintf(out,"\n");
	}
	fclose(out);
}
}

void light_dump_1d(struct light *in, int i,char *ext)
{
char out_dir[1024];
if ((get_dump_status(dump_optics)==TRUE)&&(in->sun_E[i]!=0.0))
{

	sprintf(out_dir,"%s/light_dump/",in->output_path);
	struct stat st = {0};

	if (stat(out_dir, &st) == -1)
	{

	    mkdir(out_dir, 0700);
	}

	FILE *out;
	FILE *out2;
	int ii;
	char name[400];
	double max=0.0;

	if (i==0)
	{
		sprintf(name,"%s/light_1d_photons_tot%s.dat",out_dir,ext);
		out=fopen(name,"w");
		sprintf(name,"%s/light_1d_photons_tot_norm%s.dat",out_dir,ext);
		out2=fopen(name,"w");
		max=inter_array_get_max(in->photons_tot,in->points);
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii]-in->device_start,in->photons_tot[ii]);
			fprintf(out2,"%le %le\n",in->x[ii]-in->device_start,in->photons_tot[ii]/max);
		}
		fclose(out);
		fclose(out2);
	}

	sprintf(name,"%s/light_1d_%.0f_photons%s.dat",out_dir,in->l[i]*1e9,ext);
	out=fopen(name,"w");
	sprintf(name,"%s/light_1d_%.0f_photons%s_norm.dat",out_dir,in->l[i]*1e9,ext);
	out2=fopen(name,"w");
	max=inter_array_get_max(in->photons[i],in->points);
	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le\n",in->x[ii],in->photons[i][ii]);
		fprintf(out2,"%le %le\n",in->x[ii]-in->device_start,in->photons[i][ii]/max);
	}
	fclose(out);
	fclose(out2);

	sprintf(name,"%s/light_1d_%.0f_Ep%s.dat",out_dir,in->l[i]*1e9,ext);
	out=fopen(name,"w");
	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le %le %le\n",in->x[ii],pow(pow(in->Ep[i][ii],2.0)+pow(in->Epz[i][ii],2.0),0.5),in->Ep[i][ii],in->Epz[i][ii]);
	}
	fclose(out);

	sprintf(name,"%s/light_1d_%.0f_En%s.dat",out_dir,in->l[i]*1e9,ext);
	out=fopen(name,"w");
	for (ii=0;ii<in->points;ii++)
	{
		fprintf(out,"%le %le %le %le\n",in->x[ii],pow(pow(in->En[i][ii],2.0)+pow(in->Enz[i][ii],2.0),0.5),in->En[i][ii],in->Enz[i][ii]);
	}
	fclose(out);

	sprintf(name,"%s/light_1d_%.0f_pointing%s.dat",out_dir,in->l[i]*1e9,ext);
 	write_x_y_to_file(name,in->x,in->pointing_vector[i],in->points);

	sprintf(name,"%s/light_1d_%.0f_E_tot%s.dat",out_dir,in->l[i]*1e9,ext);
	write_x_y_z_to_file(name,in->x,in->E_tot_r[i],in->E_tot_i[i],in->points);

	if (get_dump_status(dump_optics_verbose)==TRUE)
	{
		sprintf(name,"%s/light_1d_%.0f_layer%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %d\n",in->x[ii],in->layer[ii]);
		}
		fclose(out);

		sprintf(name,"%s/light_1d_%.0f_Gn%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		sprintf(name,"%s/light_1d_%.0f_Gn%s_norm.dat",out_dir,in->l[i]*1e9,ext);
		out2=fopen(name,"w");
		max=inter_array_get_max(in->Gn,in->points);
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii]-in->device_start,in->Gn[ii]);
			fprintf(out2,"%le %le\n",in->x[ii]-in->device_start,in->Gn[ii]/max);
		}
		fclose(out);
		fclose(out2);


		sprintf(name,"%s/light_1d_%.0f_Gp%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],in->Gp[ii]);
		}
		fclose(out);





		sprintf(name,"%s/light_1d_%.0f_E%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],pow(pow(in->Ep[i][ii]+in->En[i][ii],2.0)+pow(in->Enz[i][ii]+in->Epz[i][ii],2.0),0.5));
		}
		fclose(out);



		sprintf(name,"%s/light_1d_%.0f_t%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],cabs(in->t[i][ii]));
		}
		fclose(out);

		sprintf(name,"%s/light_1d_%.0f_r%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],cabs(in->r[i][ii]));
		}
		fclose(out);

		sprintf(name,"%s/light_1d_%.0f_photons_abs%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");
		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],in->photons_asb[i][ii]);
		}
		fclose(out);


		sprintf(name,"%s/light_1d_%.0f_n%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");

		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],in->n[i][ii]);
		}


		fclose(out);

		sprintf(name,"%s/light_1d_%.0f_alpha%s.dat",out_dir,in->l[i]*1e9,ext);
		out=fopen(name,"w");

		for (ii=0;ii<in->points;ii++)
		{
			fprintf(out,"%le %le\n",in->x[ii],in->alpha[i][ii]);
		}


		fclose(out);


		out=fopena(out_dir,"light_sun_wavelength_E.dat","w");
		for (ii=0;ii<in->lpoints;ii++)
		{
			fprintf(out,"%le %le\n",in->l[ii],in->sun_E[ii]);
		}
		fclose(out);

	}


}

}

/*void light_do_stats()
{
double tot=0.0;

	int stored=FALSE;
	tot=0.0;
	if (in->gather==TRUE)
	{
		for (i=0;i<cell->ymeshpoints;i++)
		{
			if (cell->imat[i]==0)
			{
				if (fabs(cell->ymesh[i]-cell->mat.l[0].height)<in->dlength)
				{
					tot+=cell->Gn[i];
					printf("yes\n");
				}else
				{
					printf("no\n");
				}
				cell->Gn[i]=0.0;
				cell->Gp[i]=0.0;
				stored=TRUE;
			}else
			{
				if (stored==TRUE)
				{
					if ((i-1)>=0)
					{
						cell->Gp[i-1]=tot;
					}else
					{
						cell->Gp[i]+=tot;
					}

					cell->Gn[i+1]+=tot;
					tot=0.0;
					stored=FALSE;
				}
			}
		}
	}

	for (i=0;i<in->points;i++)
	{
		if (in->enable_gen[in->layer[i]]==0)
		{
			in->Gn[i]=0.0;
			in->Gp[i]=0.0;
		}

	}

	for (i=0;i<in->layers;i++)
	{
		in->G_percent[i]=0.0;
	}

	tot=0.0;
	for (i=0;i<in->points;i++)
	{
		in->G_percent[in->layer[i]]+=in->Gn[i];
		tot+=in->Gn[i];
	}

	for (i=0;i<in->layers;i++)
	{
		in->G_percent[i]*=100.0/tot;
		if (get_dump_status(dump_optics)==TRUE)
		{
			if (tot!=0.0) printf("%d %lf\n",i,in->G_percent[i]);
		}
	}

	if (get_dump_status(dump_optics)==TRUE)
	{
		FILE *out;
		out=fopena(out_dir,"light_device_G.dat","w");
		for (i=0;i<cell->ymeshpoints;i++)
		{
			fprintf(out,"%le %le %le\n",cell->ymesh[i],cell->Gn[i],cell->Gp[i]);
		}
		fclose(out);
	}
}*/
