// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include <math.h>
#include <util.h>
#include <i.h>
#include <gsl/gsl_multimin.h>
#include "fit.h"

static int unused __attribute__((unused));

double fit_load_plugin(struct fitvars *config,char *base_name,int ext)
{
void *lib_handle;
double (*fn)(struct fitvars *,int,char *);
char *error;

char libname[100];

sprintf(libname,"./exp/%s%d/error_plugin.so",base_name,ext);

   lib_handle = dlopen(libname, RTLD_LAZY);
   if (!lib_handle)
   {
      fprintf(stderr, "%s\n", dlerror());
   }

   fn = dlsym(lib_handle, "lib_error");
   if ((error = dlerror()) != NULL)
   {
      fprintf(stderr, "%s\n", error);
   }

   double ret=(*fn)(config,ext,"");

   dlclose(lib_handle);
   return ret;
}


double get_voc_tau_error(struct fitvars *config)
{
double a,b;
printf("get_tau_error\n");

FILE *in;
FILE *out;

out=fopen("./sim/voc_tau.dat","w");

in=fopen("./sim/jv1/voc_tau.dat","r");
unused=fscanf(in,"%le %le",&a,&b);
fprintf(out,"%le %le\n",a,b);
fclose(in);

in=fopen("./sim/jv2/voc_tau.dat","r");
unused=fscanf(in,"%le %le",&a,&b);
fprintf(out,"%le %le\n",a,b);
fclose(in);

fclose(out);


char expfile[1000];
char simfile[1000];
char out_name[1000];
strcpy(expfile,"./exp/voc_tau/voc_tau.dat");
strcpy(simfile,"./sim/voc_tau.dat");
sprintf(out_name,"./error_voc_tau.dat");

double error=0.0;
struct istruct exp;
struct istruct sim;
inter_load_a(&exp,expfile,0,1);
inter_load_a(&sim,simfile,0,1);
inter_sort(&sim);

double start=0.0;
double stop=0.0;
char tempstring[100];
double max=0.0;

	FILE *info=fopena("./exp/voc_tau/","fit.inp","r");
	if (info==NULL)
	{
		ewe("Problem with ./exp/voc_tau/fit.inp\n");
	}
	unused=fscanf(info,"%s",tempstring);
	unused=fscanf(info,"%le",&start);
	unused=fscanf(info,"%s",tempstring);
	unused=fscanf(info,"%le",&stop);
	unused=fscanf(info,"%s",tempstring);

	if (strcmp("#end",tempstring)!=0)
	{
		ewe("Problem with fit.inp in %s!\n","./exp/voc_tau/");
	}
	fclose(info);
start=exp.data[0];
if (sim.x[0]>start) start=sim.x[0];
if (exp.x[0]>start) start=exp.x[0];


inter_log_y(&sim);
inter_log_y(&exp);




double min=exp.data[exp.len-1];

inter_sub_double(&exp,min);
inter_sub_double(&sim,min);




max=exp.data[0];//inter_get(&exp,start);//inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&exp,1.0, 1.0/max);
inter_rescale(&sim,1.0, 1.0/max);


error=inter_get_diff(NULL,&exp,&sim,start,stop,config->voc_tau_mul);

inter_free(&exp);
inter_free(&sim);
return error;
}

double get_charge_light_ss_error(struct fitvars *config)
{
printf("get ce light ss error\n");
double error=0.0;
double max=0.0;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/celight/ce.dat");
strcpy(sim_name,"./sim/jv1/charge.dat");
strcpy(out_name,"./error_ce_light.dat");

struct istruct exp;
struct istruct sim;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);


inter_log_y(&sim);
inter_log_y(&exp);

double start=exp.x[1];
if (sim.x[0]>start) start=sim.x[0];
start=0.3;
double stop=0.6;

double min=inter_get(&exp,start);
inter_sub_double(&exp,min);
inter_sub_double(&sim,min);


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);


FILE *ferror;
ferror=fopen(out_name,"w");


error=inter_get_diff(NULL,&exp,&sim,start,stop,config->chargedark_fitmul);

fclose(ferror);

inter_free(&exp);
inter_free(&sim);

return error;
}


double get_charge_ss_error(struct fitvars *config)
{
printf("get ce ss error\n");
double error=0.0;
double max=0.0;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/cedark/ce.dat");
strcpy(sim_name,"./sim/jv0/charge.dat");
strcpy(out_name,"./error_ce.dat");

struct istruct exp;
struct istruct sim;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);


inter_log_y(&sim);
inter_log_y(&exp);

double start=0.4;
if (sim.x[0]>start) start=sim.x[0];
if (exp.x[0]>start) start=exp.x[0];

double stop=0.6;
if (sim.x[sim.len-1]<stop) stop=sim.x[sim.len-1];
if (exp.x[exp.len-1]<stop) stop=exp.x[exp.len-1];

double min=inter_get(&exp,start);
inter_sub_double(&exp,min);
inter_sub_double(&sim,min);



max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);



FILE *ferror;
ferror=fopen(out_name,"w");


error=inter_get_diff(NULL,&exp,&sim,start,stop,config->chargedark_fitmul);

fclose(ferror);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_charge_error(struct fitvars *config)
{
printf("get jv error\n");
double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct sim;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/cedata.dat");
strcpy(sim_name,"./sim/ce/realce.dat");
strcpy(out_name,"./error_ce.dat");

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);
//inter_save(&exp,"./a.dat");
//inter_save(&sim,"./b.dat");
//getchar();
//inter_dump(&sim);

inter_log_y(&sim);
inter_log_y(&exp);

double min=exp.data[0];
inter_sub_double(&sim,min);
inter_sub_double(&exp,min);


max=inter_norm_to_one(&exp);
inter_rescale(&sim,1.0, 1.0/max);

double start=exp.x[1];
double stop=exp.x[exp.len-1];

if (sim.x[0]>start) start=sim.x[0];
error=inter_get_diff(NULL,&exp,&sim,start,stop,config->chargedark_fitmul);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_charge_error_light(struct fitvars *config)
{
printf("get jv error\n");
double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct sim;
char exp_name[40];
char sim_name[40];
char out_name[40];
strcpy(exp_name,"./exp/celight.dat");
strcpy(sim_name,"./sim/celight/realce.dat");
strcpy(out_name,"./error_ce_light.dat");

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);
//inter_save(&exp,"./a.dat");
//inter_save(&sim,"./b.dat");
//getchar();
//inter_dump(&sim);

inter_log_y(&sim);
inter_log_y(&exp);

double start=exp.x[1];
double stop=exp.x[exp.len-1];
double min;

if (sim.x[0]>start) start=sim.x[0];

min=exp.data[0];

inter_sub_double(&sim,min);
inter_sub_double(&exp,min);


max=inter_norm_to_one(&exp);
inter_rescale(&sim,1.0, 1.0/max);


error=inter_get_diff(NULL,&exp,&sim,start,stop,config->chargedark_fitmul);

inter_free(&exp);
inter_free(&sim);

return error;
}

double get_ce_trans_error(struct fitvars *config,int ext)
{
//printf("get_ce_trans_error\n");
char exp_name[40];
char sim_name[40];
char config_name[40];
char out_name[40];
char temp[100];
double chop;
sprintf(exp_name,"./exp/ce%d/ce.dat",ext);
sprintf(sim_name,"./sim/ce%d/cei.dat",ext);
sprintf(out_name,"./error_ce_trans%d.dat",ext);
sprintf(config_name,"./exp/ce%d/cefit.inp",ext);

double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct simtime;
double start=-6.7;
double stop=-4;
double error_ce_trans_mull=1.0;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&simtime,sim_name,0,1);

FILE *file_config;
if ((file_config=fopen(config_name,"r"))==NULL)
{
    ewe("Error opening file: %s.\n", strerror(errno));
}

unused=fscanf(file_config,"%s",temp);
unused=fscanf(file_config,"%le",&(chop));
unused=fscanf(file_config,"%s",temp);
unused=fscanf(file_config,"%le",&(start));
unused=fscanf(file_config,"%s",temp);
unused=fscanf(file_config,"%le",&(stop));
unused=fscanf(file_config,"%s",temp);
unused=fscanf(file_config,"%le",&(error_ce_trans_mull));
fclose(file_config);

inter_chop(&exp,chop, 1.0);
inter_chop(&simtime,chop, 1.0);

//inter_save(&exp,"./a.dat");
//inter_save(&simtime,"./b.dat");

double min=inter_get_min(&simtime);

inter_sub_double(&simtime,min);
inter_sub_double(&exp,min);

inter_purge_zero(&simtime);
inter_purge_zero(&exp);

inter_log_y(&simtime);
inter_log_y(&exp);
inter_log_x(&simtime);
inter_log_x(&exp);

inter_chop(&exp,start, stop);
inter_chop(&simtime,start, stop);


min=exp.data[exp.len-1];

inter_sub_double(&simtime,min);
inter_sub_double(&exp,min);

start=exp.x[0];
if (start<simtime.x[0]) start=simtime.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&simtime,1.0, 1.0/max);

error=inter_get_diff(NULL,&exp,&simtime,start,stop,error_ce_trans_mull);

inter_free(&exp);
inter_free(&simtime);
return error;
}

double get_tpv_error(struct fitvars *config,int ext)
{
char exp_name[40];
char sim_name[40];
char out_name[40];
sprintf(exp_name,"./exp/tpv%d/tpv_voc.dat",ext);
sprintf(sim_name,"./sim/tpv%d/tpv_voc_norm.dat",ext);
sprintf(out_name,"./error_tpv%d.dat",ext);


struct istruct exp;
struct istruct sim;
double start=0.0;
double stop=0.0;
double error=0.0;
double max=0.0;
inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);

double min=inter_get_min(&sim);

inter_sub_double(&sim,min-1e-9);
inter_sub_double(&exp,min-1e-9);

inter_purge_zero(&sim);
inter_purge_zero(&exp);
min=exp.data[exp.len-1];
inter_save(&exp,"./a.dat");
inter_save(&sim,"./b.dat");
inter_sub_double(&sim,min);
inter_sub_double(&exp,min);

start=exp.x[0];
if (start<sim.x[0]) start=sim.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);



error=inter_get_diff(NULL,&exp,&sim,start,stop,1.0);

inter_free(&exp);
inter_free(&sim);
printf("TPV returned %le\n",error);
return error;
}

double get_starknp_error(struct fitvars *config,int ext)
{
FILE *file;
char exp_name[40];
char sim_name[40];
char out_name[40];
char out_name_a[40];
char out_name_b[40];
char fit_config_name[40];
double div_point;
double mul_left;
double mul_right;
double ver;
char name[40];
sprintf(exp_name,"./exp/stark%d/stark.dat",ext);
sprintf(sim_name,"./sim/stark%d/starknp.dat",ext);
sprintf(fit_config_name,"./exp/stark%d/fit.inp",ext);
sprintf(out_name,"./error_starknp_trans%d.dat",ext);
sprintf(out_name_a,"./error_starknp_trans_a%d.dat",ext);
sprintf(out_name_b,"./error_starknp_trans_b%d.dat",ext);


file=fopen(fit_config_name,"r");
unused=fscanf(file,"%s",name);
unused=fscanf(file,"%le",&(div_point));
unused=fscanf(file,"%s",name);
unused=fscanf(file,"%le",&(mul_left));
unused=fscanf(file,"%s",name);
unused=fscanf(file,"%le",&(mul_right));

unused=fscanf(file,"%s",name);
unused=fscanf(file,"%lf",&(ver));

if (ver!=1.0)
{
	ewe("File compatability problem %s\n",fit_config_name);
}

unused=fscanf(file,"%s",name);
if (strcmp("#end",name)!=0)
{
	ewe("Problem with %s file!\n",fit_config_name);
}
fclose(file);

struct istruct exp;
struct istruct sim;
double start=0.0;
double stop=0.0;
double error=0.0;
double max=0.0;
inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim,sim_name,0,1);
inter_sort(&exp);

double min=inter_get_min(&sim);

inter_sub_double(&sim,min);
inter_sub_double(&exp,min);

inter_purge_zero(&sim);
inter_purge_zero(&exp);
min=exp.data[0];

inter_sub_double(&sim,min);
inter_sub_double(&exp,min);

start=exp.x[0];
if (start<sim.x[0]) start=sim.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);

inter_save(&exp,"./a.dat");
inter_save(&sim,"./b.dat");

error=inter_get_diff(NULL,&exp,&sim,start,div_point,mul_left);
error+=inter_get_diff(NULL,&exp,&sim,div_point,stop,mul_right);
printf("Roderick - %le %le %le\n",div_point,mul_left,mul_right);
inter_free(&exp);
inter_free(&sim);
return error;
}

double get_celiv_error(struct fitvars *config,int ext)
{
return fit_load_plugin(config,"celiv",ext);
}

double get_stark_error(struct fitvars *config,int ext)
{
return fit_load_plugin(config,"stark",ext);
}

double get_shg_error(struct fitvars *config,int ext)
{
char exp_name[40];
char sim_on_name[40];
char sim_off_name[40];
char out_name[40];
char sim_on_name_out[40];

sprintf(exp_name,"./exp/shg%d/shg.dat",ext);
sprintf(sim_on_name,"./sim/shg_on_%d/shg_stark.dat",ext);
sprintf(sim_off_name,"./sim/shg_off_%d/shg_stark.dat",ext);
sprintf(out_name,"./error_shg_trans%d.dat",ext);
sprintf(sim_on_name_out,"./sim/shg_on_%d/shg_shg.dat",ext);

struct istruct exp;
struct istruct sim_on;
struct istruct sim_off;

double start=0.0;
double stop=0.0;
double error=0.0;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&sim_on,sim_on_name,0,1);
inter_load_a(&sim_off,sim_off_name,0,1);

inter_div(&sim_on,&sim_off);
inter_save(&sim_on,sim_on_name_out);

inter_sort(&exp);

inter_purge_zero(&sim_on);
inter_purge_zero(&sim_off);

inter_purge_zero(&exp);

double min=1.0-exp.data[0];

inter_sub_double(&sim_on,min);
inter_sub_double(&exp,min);

start=exp.x[0];
if (start<sim_on.x[0]) start=sim_on.x[0];
stop=exp.x[exp.len-1];
if (stop>sim_on.x[sim_on.len-1]) stop=sim_on.x[sim_on.len-1];

//min=-1.0*inter_get_min_range(&exp,start,stop);

inter_save(&exp,"./a.dat");
inter_save(&sim_on,"./b.dat");
//getchar();
//inter_rescale(&sim,1.0, 1.0/min);
//inter_rescale(&exp,1.0, 1.0/min);


error=inter_get_diff(NULL,&exp,&sim_on,start,stop,1.0);

inter_free(&exp);
inter_free(&sim_on);
inter_free(&sim_off);
return error;
}

double get_pulse_error(struct fitvars *config,int ext)
{
return fit_load_plugin(config,"pulse",ext);
}

double get_pulse_voc_error(struct fitvars *config,int ext)
{
return fit_load_plugin(config,"pulse_voc",ext);
}

double get_tof_error(struct fitvars *config,int ext)
{
printf("get_tof_error\n");
char exp_name[40];
char sim_name[40];
char config_name[40];
char out_name[40];
char temp[100];
double chop;
sprintf(exp_name,"./exp/tof%d/tof.dat",ext);
sprintf(sim_name,"./sim/tof%d/tof.dat",ext);
sprintf(out_name,"./error_tof%d.dat",ext);
sprintf(config_name,"./exp/tof%d/toffit.inp",ext);

double error=0.0;
double max=0.0;
struct istruct exp;
struct istruct simtime;
double start=-6.7;
double stop=-4;

inter_load_a(&exp,exp_name,0,1);
inter_load_a(&simtime,sim_name,0,1);

FILE *file_config;
if ((file_config=fopen(config_name,"r"))==NULL)
{
    ewe("Error opening file: %s.\n", strerror(errno));
}

unused=fscanf(file_config,"%s",temp);
unused=fscanf(file_config,"%le",&(chop));
unused=fscanf(file_config,"%s",temp);
unused=fscanf(file_config,"%le",&(start));
unused=fscanf(file_config,"%s",temp);
unused=fscanf(file_config,"%le",&(stop));


inter_chop(&exp,chop, 1.0);
inter_chop(&simtime,chop, 1.0);


fclose(file_config);

double min=inter_get_min(&simtime);

inter_sub_double(&simtime,min);
inter_sub_double(&exp,min);

inter_purge_zero(&simtime);
inter_purge_zero(&exp);

inter_log_y(&simtime);
inter_log_y(&exp);
inter_log_x(&simtime);
inter_log_x(&exp);


min=exp.data[exp.len-1];

inter_sub_double(&simtime,min);
inter_sub_double(&exp,min);

start=exp.x[0];
if (start<simtime.x[0]) start=simtime.x[0];
stop=exp.x[exp.len-1];


max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&simtime,1.0, 1.0/max);


error=inter_get_diff(NULL,&exp,&simtime,start,stop,config->tof_error_mull);

inter_free(&exp);
inter_free(&simtime);
printf("get_tof_error ret= %d %e %e %e\n",ext,error,min,max );
return error;
}


double get_tpc_error(struct fitvars *config,int ext)
{
return fit_load_plugin(config,"tpc",ext);
}

double get_jvtau_error(struct fitvars *config)
{
double exp_tau;
double sim_tau;
double temp;
double error;

printf("get_jvtau_error\n");
FILE *exp=fopen("./exp/jvlight/voctaumin.inp","r");
unused=fscanf(exp,"%le",&exp_tau);
fclose(exp);

FILE *sim=fopen("./sim/jv1/voc_tau.dat","r");
unused=fscanf(sim,"%le",&temp);
unused=fscanf(sim,"%le",&sim_tau);
fclose(sim);


error=fabs(((sim_tau/exp_tau)-1.0)*0.5);

if (sim_tau>exp_tau) error=0.0;

return error;
}

double get_vocdarkmu(struct fitvars *config)
{
double mu_exp_e;
double mu_exp_h;
double mu_sim_e;
double mu_sim_h;
double error;

printf("get_vocdarkmu_error\n");
FILE *exp=fopen("./exp/vocdarkmu/data.dat","r");
unused=fscanf(exp,"%le",&mu_exp_e);
unused=fscanf(exp,"%le",&mu_exp_h);
fclose(exp);

FILE *sim=fopen("./sim/jv1/voc_mue.dat","r");
unused=fscanf(sim,"%le",&mu_sim_e);
fclose(sim);

sim=fopen("./sim/jv1/voc_muh.dat","r");
unused=fscanf(sim,"%le",&mu_sim_h);
fclose(sim);

mu_exp_e=fabs(log10(mu_exp_e));
mu_exp_h=fabs(log10(mu_exp_h));
mu_sim_e=fabs(log10(mu_sim_e));
mu_sim_h=fabs(log10(mu_sim_h));

error=fabs(((mu_sim_e/mu_exp_e)-1.0)*0.5);
error+=fabs(((mu_sim_h/mu_exp_h)-1.0)*0.5);

return error;
}

double get_jv_error(struct fitvars *config,int ext)
{
return fit_load_plugin(config,"jv",ext);
}

double get_band_error(struct fitvars *config)
{
int dostype=read_value("dos.inp",0,2);
double Ee=0.0;
double Eh=0.0;
double ret=0.0;
if (dostype==0)
{
	Ee=read_value("dos.inp",0,38);
	Eh=read_value("dos.inp",0,40);
	ret=1+tanh( (Ee-Eh)*20.0/Ee);
	ret*=config->banderror_fitmul;
}else
{

	Ee=read_value("dos.inp",0,12);
	Eh=read_value("dos.inp",0,14);
	ret=1+tanh( (Ee-Eh)*10.0/Ee);
	ret*=config->banderror_fitmul;
}

return ret;
}

