// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../dump.h"
#include "tof.h"

double newton_ittr_tof_step(struct device *in,int usecap)
{
double clamp=0.1;
double step=0.01;
double e0;
double e1;
double i0;
double i1;
double deriv;
double C=in->C;
if (usecap==FALSE) C=0.0;

double Rs=in->Rcontact+tof_config.tof_Rshort;

newton_sim_tof(in);
i0=get_I(in);

double itot=i0+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt;

e0=fabs(itot*Rs+in->Vapplied-tof_config.tof_power_suply);

in->Vapplied+=step;
newton_sim_tof(in);

i1=get_I(in);
itot=i1+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt;

e1=fabs(itot*Rs+in->Vapplied-tof_config.tof_power_suply);
//printf("%le\n",e1);
deriv=(e1-e0)/step;
step=-e1/deriv;
//step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
int count=0;
int max=1000;
do
{
e0=e1;
newton_sim_tof(in);
i1=get_I(in);
itot=i1+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt;
e1=fabs(itot*Rs+in->Vapplied-tof_config.tof_power_suply);
//printf("error=%le Vapplied=%le \n",e1,in->Vapplied);
deriv=(e1-e0)/step;
step=-e1/deriv;
//double clamp=0.01;
//if (e1<clamp) clamp=e1/100.0;
//step=step/(1.0+fabs(step/clamp));
step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
if (count>max) break;
count++;
}while(e1>1e-11);


double ret=get_I(in)+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt;
//getchar();
return ret;
}

#ifdef enable_clever_contacts
void newton_aux_tof(struct device *in,double V,double* i,double* didv,double* didphi,double* didxil,double* didxipl,double* didphir,double* didxir,double* didxipr)
{
double i0=*i;
double didv0=*didv;
double didphi0=*didphi;
double didxil0=*didxil;
double didxipl0=*didxipl;
double didphir0=*didphir;
double didxir0=*didxir;
double didxipr0=*didxipr;
double itot=i0+V/in->Rshunt;
double ditotdv=didv0+1.0/in->Rshunt;
*i=tof_config.tof_power_suply-(V+(in->Rcontact+tof_config.tof_Rshort)*(itot));
*didv=-(1.0+(in->Rcontact+tof_config.tof_Rshort)*(ditotdv));
*didphi=-((in->Rcontact+tof_config.tof_Rshort)*(didphi0));
*didxil=-((in->Rcontact+tof_config.tof_Rshort)*(didxil0));
*didxipl=-((in->Rcontact+tof_config.tof_Rshort)*(didxipl0));
*didphir=-((in->Rcontact+tof_config.tof_Rshort)*(didphir0));
*didxir=-((in->Rcontact+tof_config.tof_Rshort)*(didxir0));
*didxipr=-((in->Rcontact+tof_config.tof_Rshort)*(didxipr0));
return;
}
#endif
double newton_sim_tof_all(struct device *in,int usecap)
{
//#ifdef enable_clever_contacts
//in->newton_cur=TRUE;
//solver_realloc(in);
//in->newton_aux=&newton_aux_tof;
//solve_all(in);//in->C*(in->Vapplied-in->Vapplied_last)/in->dt+
//return get_I(in)+in->Vapplied/in->Rshunt;
//#else
return newton_ittr_tof_step(in,usecap);
//#endif
}

void newton_sim_tof(struct device *in)
{
in->newton_cur=FALSE;
solver_realloc(in);
solve_all(in);
}

