// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../slice.h"
#include "../../dump.h"
#include "../../ntricks.h"
#include "pulse_voc.h"
#include "../../gui_hooks.h"

static int unused __attribute__((unused));

struct pulse_voc pulse_voc_config;

void sim_pulse_voc(struct device *in)
{
gui_start();
double V=0.0;
pulse_voc_load_config(&pulse_voc_config,in);

in->dt=pulse_voc_config.pulse_voc_dt;



in->go_time=FALSE;


in->externalv=pulse_voc_config.pulse_voc_Vexternal;
printf("Running pulse_voc. %e\n",in->externalv);
int ittr=0;

int step=0;
light_solve_and_update(in,&(in->mylight),pulse_voc_config.pulse_voc_light_bias,0.0);
sim_externalv(in,in->externalv);

remesh_reset(in,in->externalv);

light_solve_and_update(in,&(in->mylight),pulse_voc_config.pulse_voc_light_bias,0.0);

newton_pulse_voc(in,FALSE);
device_timestep(in);

newton_pulse_voc(in,FALSE);
device_timestep(in);

in->time=0.0;

time_init(in);

device_timestep(in);

in->go_time=TRUE;


double t0=pulse_voc_config.pulse_voc_t0;
double t1=pulse_voc_config.pulse_voc_t0+pulse_voc_config.pulse_voc_t1;
double t2=pulse_voc_config.pulse_voc_t0+pulse_voc_config.pulse_voc_t1+pulse_voc_config.pulse_voc_t2;
FILE *pulse_voc_out=fopen("./pulse_voc_j.dat","w");
fclose(pulse_voc_out);

pulse_voc_out=fopen("./pulse_voc_v.dat","w");
fclose(pulse_voc_out);

pulse_voc_out=fopen("./pulse_voc_nleft.dat","w");
fclose(pulse_voc_out);

pulse_voc_out=fopen("./pulse_voc_rtot.dat","w");
fclose(pulse_voc_out);

double local_time=in->time;

dump_dynamic_init(in);

struct istruct out_i;
inter_init(&out_i);

struct istruct out_v;
inter_init(&out_v);

struct istruct out_G;
inter_init(&out_G);

struct istruct lost_charge;
inter_init(&lost_charge);


double extracted_through_contacts=0.0;
carrier_count_reset(in);
reset_npinit(in);
do
{



	if (local_time<t0)
	{
		light_solve_and_update(in,&(in->mylight),pulse_voc_config.pulse_voc_light_bias,0.0);

		extracted_through_contacts=0.0;
	}else
	if (local_time<t1)
	{
		light_solve_and_update(in,&(in->mylight),pulse_voc_config.pulse_voc_light_bias,pulse_voc_config.pulse_voc_laser_eff);
	}else
	{
		light_solve_and_update(in,&(in->mylight),pulse_voc_config.pulse_voc_light_bias,0.0);
	}

	dump_dynamic_add_data(in);
	double i0=newton_pulse_voc(in,TRUE);
	V=in->Vapplied;
	//printf ("%le %le %le\n",in->externalv,in->Vapplied,V);
	//getchar();
	if (get_dump_status(dump_iodump)==TRUE)
	{
		printf("pulse_voc time=%e step=%d %.1e ",in->time,step,in->last_error);
		printf("Vtot=%lf current = %e mA (%e A/m^2)\n",V,get_I(in)/1e-3,get_J(in));
	}

	ittr++;

	slice_check(in);

	plot_now(in,"pulse_voc.plot");


	inter_append(&out_i,in->time,i0);
	inter_append(&out_v,in->time,V);
	inter_append(&out_G,in->time,in->Gn[0]);
	inter_append(&lost_charge,in->time,extracted_through_contacts-fabs(get_extracted_n(in)+get_extracted_p(in))/2.0);

	if (pulse_voc_config.pulse_voc_use_big_mesh==TRUE)
	{
		if ((in->time>pulse_voc_config.pulse_voc_bigmeshstart)&&(in->time<pulse_voc_config.pulse_voc_bigmeshstop))
		{
			in->dt=pulse_voc_config.pulse_voc_bigdt;
		}else
		{
			in->dt=pulse_voc_config.pulse_voc_dt;
		}
	}
	device_timestep(in);
	step++;

	if (in->time>(((double)pulse_voc_config.pulse_voc_repeat)*t2)) break;
	if (local_time>t2) local_time=0.0;


	local_time+=in->dt;

	if (in->stop==TRUE)
	{
		break;

	}

	stop_start(in);

}while(1);
char outpath[1000];
struct istruct out_flip;

dump_dynamic_save(in);
dump_dynamic_free(in);

inter_copy(&out_flip,&out_i,TRUE);
inter_mul(&out_flip,-1.0);
sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_i_pos.dat");
inter_save(&out_flip,outpath);

inter_free(&out_flip);


sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_i.dat");
inter_save(&out_i,outpath);

inter_add_x(&out_i,-t0+pulse_voc_config.pulse_voc_shift);
sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_i_norm.dat");
inter_mod(&out_i);
inter_save(&out_i,outpath);


inter_sub_double(&out_v,out_v.data[0]);
sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_v.dat");
inter_save(&out_v,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_G.dat");
inter_save(&out_G,outpath);

sprintf(outpath,"%s%s",in->outputpath,"pulse_voc_lost_charge.dat");
inter_save(&lost_charge,outpath);



in->go_time=FALSE;

inter_free(&out_G);
inter_free(&out_i);
inter_free(&out_v);
//</clean>
gui_stop();
}

void pulse_voc_load_config(struct pulse_voc *in,struct device *dev)
{
double ver=0.0;
char name[200];
FILE *config=fopena(dev->outputpath,"pulse_voc.inp","r");
if (config==NULL)
{
ewe("Error opening file pulse_voc.inp");
}
config_read_line_to_double(&(in->pulse_voc_dt),config,"#pulse_voc_dt");

config_read_line_to_double(&(in->pulse_voc_t0),config,"#pulse_voc_t0");

config_read_line_to_double(&(in->pulse_voc_t1),config,"#pulse_voc_t1");

config_read_line_to_double(&(in->pulse_voc_t2),config,"#pulse_voc_t2");

config_read_line_to_double(&(in->pulse_voc_Vexternal),config,"#pulse_voc_Vexternal");

config_read_line_to_int(&(in->pulse_voc_repeat),config,"#pulse_voc_repeat");

config_read_line_to_double(&(in->Rshort_pulse_voc),config,"#Rshort_pulse_voc");

config_read_line_to_double(&(in->pulse_voc_laser_eff),config,"#pulse_voc_laser_eff");
in->pulse_voc_laser_eff=fabs(in->pulse_voc_laser_eff);

config_read_line_to_double(&(in->pulse_voc_shift),config,"#pulse_voc_shift");

config_read_line_to_int(&(in->pulse_voc_use_big_mesh),config,"#pulse_voc_use_big_mesh");

config_read_line_to_double(&(in->pulse_voc_bigmeshstart),config,"#pulse_voc_bigmeshstart");

config_read_line_to_double(&(in->pulse_voc_bigmeshstop),config,"#pulse_voc_bigmeshstop");

config_read_line_to_double(&(in->pulse_voc_bigdt),config,"#pulse_voc_bigdt");

config_read_line_to_double(&(in->pulse_voc_light_bias),config,"#pulse_voc_light_bias");
in->pulse_voc_light_bias=fabs(in->pulse_voc_light_bias);

config_read_line_to_double(&(ver),config,"#ver");

if (ver!=1.24)
{
ewe("File compatability problem %s\n","pulse_voc.inp");
}

unused=fscanf(config,"%s",name);
if (strcmp("#end",name)!=0)
{
ewe("Problem with pulse_voc.inp file last item read %s!\n",name);
}
fclose(config);
}
