// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../sim.h"
#include "../../ntricks.h"
#include "qe.h"

void sim_qe(struct device *in)
{
newton_set_min_ittr(30);
in->Vapplied=0.0;
sim_externalv(in,in->Vapplied);
//newton_sim_jv(in);
newton_set_min_ittr(0);
double lambda=in->mylight.lstart;
FILE *out=fopen("qe.dat","w");
fclose(out);
double Icur;
struct istruct glass;
inter_init(&glass);
inter_load_a(&glass,"./glass_tx.inp",0,1);
inter_sort(&glass);
do
{
	in->mylight.laser_wavelength=lambda;
	in->mylight.laser_pos=(int)((in->mylight.laser_wavelength-in->mylight.lstart)/in->mylight.dl);
	in->mylight.laser_photon_density=1e22*inter_get(&glass,lambda);
	light_solve_and_update(in,&(in->mylight),0.0,1e-5);

	sim_externalv(in,in->Vapplied);

	Icur=get_equiv_J(in);

	out=fopen("qe.dat","a");
	fprintf(out,"%le %le\n",lambda,fabs(Icur/q)/1e22);
	fclose(out);

	//plot_now(in,"plot");

	if (in->stop==TRUE)
	{
		break;
	}

	stop_start(in);

	lambda+=in->mylight.dl;

}while(lambda<in->mylight.lstop);
inter_free(&glass);


}

